/* 
   Algorithms IE171 - Lab2 - Selection Algorithms
   Author: Matt Galati
   Revision: Menal Guzelsoy, Ted Ralphs - Jan '04
*/

#include "select.hpp"

/*===========================================================================*/
/*====================== Constructors and Destructors =======================*/
/*===========================================================================*/

Select::Select(const char * filename):
list(0),
listCopy(0),
listSize(0),
randomSeedSet(false),
partitionStrategy(RANDOM)
{
   readList(filename);
}

/*===========================================================================*/

Select::~Select()
{
   if (list){
      delete [] list;
   }
   if (listCopy){
      delete [] listCopy;
   }
}

/*===========================================================================*/
/*=============================== Query Methods =============================*/
/*===========================================================================*/

int Select::getListSize()
{
   return listSize;
}

/*===========================================================================*/

pair<string,int> Select::getListItem(const int i)
{
   return list[i];
}

/*===========================================================================*/

Strategy Select::getStrategy()
{
   return partitionStrategy;
}

/*===========================================================================*/

bool Select::isRandomSeedSet()
{
   return randomSeedSet;
}

/*===========================================================================*/
/*============================ Modification Methods =========================*/
/*===========================================================================*/

void Select::setRandomSeed(int randSeed)
{
   srand(randSeed);
   randomSeedSet = true;
}

/*===========================================================================*/

void Select::setStrategy(Strategy strgy)
{
   partitionStrategy = strgy;
}

/*===========================================================================*/

void Select::readList(const char * filename)
{
   ifstream is (filename);
   
   if (!is) {
      cout<<"Error in reading input file"<<endl;
      exit(1);
   }
   
   //Read in the length of the list
   is >> listSize; 
   
   list  = new pair<string,int>[listSize]; //The list
   listCopy  = new pair<string,int>[listSize]; //The copy list
   
   //Read in the list
   int i(0);
   for (i = 0; i < listSize; i++){
      is >> list[i].first >> list[i].second; 
   }
   
   copy(list, list+listSize, listCopy);
   
   is.close();
}

/*===========================================================================*/

void Select::resetList()
{
   copy(listCopy, listCopy+listSize, list);
}

/*===========================================================================*/
/*============================= List Operations =============================*/
/*===========================================================================*/

pair<string,int> Select::selectInsertion(const int p, const int r, 
					 const int numSelect)
{

   //**********************************************************************
   //**********************************************************************
   // You should fill in the code for the insertion algorithm here. 
   //**********************************************************************
   //**********************************************************************

   // At the end, you should return the item in the ith position

}

/*===========================================================================*/

pair<string,int> Select::selectPartition(const int p, const int r, 
					 const int numSelect)
{

   if (p == r){
      selectInsertion(0, numSelect + p - 1, numSelect + p);
      return list[p];
   }

   int q(partition(p, r, numSelect));
   int k(q - p + 1);
   
   if (numSelect == k) {
      // If the partition element divides the array properly
      selectInsertion(0, numSelect + p - 1, numSelect + p);
      return list[q];
   }else{

   //**********************************************************************
   //**********************************************************************
   // You should fill in the recursive calls here. The
   // function that actually does the partitioning is below.  
   //**********************************************************************
   //**********************************************************************

   }

   // At the end, you should return the item in the ith position

}

/*===========================================================================*/
/*=============================== Private Methods ===========================*/
/*===========================================================================*/

int Select::partition (const int p, const int r, int numSelect)
{
   pair<string,int> partitionElement, temp;
   int i(0), j(0), k(0);      
   double r01(0.0);
   
   // This first part chooses the partition element
   // Note the partition element is always swapped to the last position
   switch(partitionStrategy){
      
    case ALWAYS_FIRST: // Always select the first element
         swap(p, r);
         break;
    case ALWAYS_LAST:  //Always select the last element
	 break;
    case ALWAYS_I:     // Always select the "ith" element
         swap(p + numSelect-1, r);
	 break;
    case RANDOM:       // Select a random element
	 if (!randomSeedSet){
	    cout << "Error! Randomized algorithm called without setting "; 
	    cout << "random seed" << endl;
	    exit(1);
	 }
         r01 = (double)rand()/RAND_MAX;
         k = (int)(p + ((r-p) * r01)); //k ~ U[p,r]
	 swap(k, r);
	 break;
    default:           // Error
	 cout<<"Error in defining partitioning strategy"<<endl;
	 exit(1);
   }

   partitionElement = list[r];

   //**********************************************************************
   //**********************************************************************
   // Here, you should fill in the code that actually does the
   // partitioning. You can use the swap function defined below.
   //**********************************************************************
   //**********************************************************************

   // At the end, you should return the new position of the partition element
   // in the partitioned array, so that you know what to do next.

}

/*===========================================================================*/

void Select::swap(const int loc1, const int loc2)
{
   pair<string, int> temp;
   temp = list[loc2]; 
   list[loc2] = list[loc1]; 
   list[loc1] = temp; 
}   

/*===========================================================================*/


